// 002 u_c_rad_html_generator
// 09 apr 2007

// -- (C) Felix John COLIBRI 2007
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_rad_html_generator;
  interface
    uses StdCtrls // tMemo
        , ExtCtrls // tPanel
        , u_c_basic_object
        , u_c_html_generator;

    type c_rad_html_generator= class(c_html_generator)
                             Constructor create_rad_html_generator(p_name: String);

                             procedure generate_page(p_c_design_panel: tPanel);

                             Destructor Destroy; Override;
                           end; // c_rad_html_generator

  implementation
    uses SysUtils, Controls
        , u_types_constants, u_characters, u_c_display, u_exe;

    // -- c_rad_html_generator

    Constructor c_rad_html_generator.create_rad_html_generator(p_name: String);
      begin
        Inherited create_html_generator(p_name);
      end; // create_rad_html_generator

    procedure c_rad_html_generator.generate_page(p_c_design_panel: tPanel);

      procedure generate_table_recursive(p_c_panel: tPanel);

        procedure generate_content;

          procedure generate_header(p_c_label: tLabel);
            begin
              with p_c_label do
                if Font.Height< -11
                  then add_line('<H3>'+ Caption+ '</H3>')
                  else add_line(Caption);
            end; // generate_header

          procedure generate_memo(p_memo_text: String);
            var l_index, l_length: Integer;
                l_result, l_anchor: String;

                l_has_bullet: Boolean;
                l_at_start_of_paragraph: Boolean;
            begin
              display('> generate_memo ');

              l_result:= '';
              l_anchor:= '';
              l_length:= Length(p_memo_text);

              l_has_bullet:= False;
              l_at_start_of_paragraph:= True;

              for l_index:= 1 to l_length do
              begin
                case p_memo_text[l_index] of
                   '<' : begin
                           if (l_index< l_length) and (p_memo_text[l_index+ 1]<> '>')
                             then l_anchor:= '<A HREF="'
                             else l_anchor:= '</A';
                         end;
                   '>' : begin
                           if l_anchor[2]= 'A'
                             then l_anchor:= l_anchor+ '">'
                             else l_anchor:= l_anchor+ '>';
                           l_result:= l_result+ l_anchor;
                           l_anchor:= '';
                         end;
                  '|' : begin
                          add_line(l_result+ '<BR>');
                          l_result:= '';
                          // l_is_empty_start_of_paragraph:= True;
                          // -- in bullet, at margin, no space => end of bullet
                          if l_has_bullet
                              and (l_index< l_length) and (p_memo_text[l_index+ 1]<> ' ')
                            then begin
                                l_result:= l_result+ '</UL>'+ k_new_line;
                                l_has_bullet:= False;
                              end;
                          l_at_start_of_paragraph:= True;
                        end;
                  '-' : begin
                          if l_at_start_of_paragraph
                            then begin
                                if not l_has_bullet
                                  then begin
                                      l_result:= l_result+ '<UL>'+ k_new_line;
                                      l_has_bullet:= True;
                                    end;
                                l_result:= l_result+ '<LI>';
                              end
                            else l_result:= l_result+ p_memo_text[l_index];
                        end;
                  chr(13), chr(10) :
                      begin
                        if l_has_bullet
                          then l_at_start_of_paragraph:= True;
                      end; // l_start_of_line:= '';

                  else
                    if l_anchor<> ''
                      then l_anchor:= l_anchor+ p_memo_text[l_index]
                      else begin
                          l_result:= l_result+ p_memo_text[l_index];
                          if p_memo_text[l_index]<> ' '
                            then begin
                                l_at_start_of_paragraph:= False;
                              end;

                        end;
                end; // case l_index
              end; // for l_index

              if l_result<> ''
                then add_line(l_result);

              display('< generate_memo');
            end; // generate_memo

          procedure generate_image(p_c_image: tImage);
            var l_file_name: String;
                l_index: Integer;
            begin
              with p_c_image do
              begin
                l_file_name:= Name;
                l_index:= Length(l_file_name);
                while (l_index> 1) and (l_file_name[l_index]<> '_') do
                  Dec(l_index);
                l_file_name[l_index]:= '.';

                add_line('<IMG SRC="'+ f_exe_path+ l_file_name+ '">');
              end;
            end; // generate_image

          procedure generate_input_box(p_c_edit: tEdit);
            begin
              // -- INPUT TYPE="text" NAME="date" VALUE="2005/5/5"
              with p_c_edit do
                add_line('<INPUT TYPE="text" NAME="'+ Name+ '" VALUE="'+ Text+ '"> <BR>');
            end; // generate_input_box

          procedure generate_submit_button(p_c_button: tButton);
            begin
              // <INPUT TYPE="submit" NAME="click" VALUE="Send"><
              with p_c_button do
                add_line('<INPUT TYPE="submit" NAME="'+ Name+ '" VALUE="'+ Caption+ '"> <BR>');
            end; // generate_submit_button

          var l_control_index: Integer;
              l_c_control_ref: tControl;

          begin // generate_content
            display('> generate_content');
            with p_c_panel do
              for l_control_index:= 0 to ControlCount- 1 do
              begin
                l_c_control_ref:= Controls[l_control_index];
                if l_c_control_ref.ClassName= 'TMemo'
                  then generate_memo(tMemo(l_c_control_ref).Text) else
                if l_c_control_ref.ClassName= 'TLabel'
                  then generate_header(tLabel(l_c_control_ref)) else
                if l_c_control_ref.ClassName= 'TImage'
                  then generate_image(tImage(l_c_control_ref)) else
                if l_c_control_ref.ClassName= 'TEdit'
                  then generate_input_box(tEdit(l_c_control_ref)) else
                if l_c_control_ref.ClassName= 'TButton'
                  then generate_submit_button(tButton(l_c_control_ref)) else
              end; // for l_control_index
            display('< generate_content');
          end; // generate_content

        var l_panel_count: Integer;
            l_panel_top_array: array of Integer;
            l_tPanel_array: array of tPanel;
            l_panel_width_array: array of Integer;
            l_total_width: Integer;
            l_color: String;
            l_color_array: array of String;

        procedure generate_the_table;
          var l_panel_index: Integer;
          begin
            // -- create the Table

            // -- either horizontal or vertical
            if l_panel_count= 1
              then begin
                  // -- <TABLE border=0 cellpadding=0 cellspacing=0 height="100%" width="100%" bgcolor="#FFBBFF" >
                  add_table_start('border=2 cellpadding=3'+ l_color);
                  add_row_start('');
                  add_cell_start('');
                  // -- recurse
                  generate_table_recursive(l_tPanel_array[0]);
                  add_cell_end;
                  add_row_end;
                end
              else
                if l_panel_top_array[0]= l_panel_top_array[1]
                  then begin
                      // -- horizontal
                      add_table_start('border=2 cellpadding=3 width="100%"'+ l_color);
                      add_row_start('');
                      for l_panel_index:= 0 to l_panel_count- 1 do
                      begin
                        // -- <TD width="20%" valign=top>
                        if l_panel_index< l_panel_count- 1
                          then add_cell_start('width="'+ IntToStr(Round((100* l_panel_width_array[l_panel_index])/ l_total_width))+ '%"'
                              + l_color_array[l_panel_index]+ ' valign=top')
                          else add_cell_start('');
                        // -- recurse
                        generate_table_recursive(l_tPanel_array[l_panel_index]);
                        add_cell_end;
                      end; // for l_panel_index
                      add_row_end;
                    end // horizontal
                  else begin
                      // -- vertical
                      add_table_start('border=2 cellpadding=3'+ l_color);
                      for l_panel_index:= 0 to l_panel_count- 1 do
                      begin
                        add_row_start('');
                        add_cell_start('');
                        // -- recurse
                        generate_table_recursive(l_tPanel_array[l_panel_index]);
                        add_cell_end;
                        add_row_end;
                      end; // for l_panel_index
                    end; // vertical
            add_table_end;
          end; // generate_table

        function f_color(p_color: Integer): String;
          begin
            Result:= IntToHex(p_color, 6);
            if Length(Result)= 6
              then Result:= ' bgcolor="#'+ Result+ '"'
              else Result:= '';
          end; // f_color

        var l_control_index: Integer;
            l_c_control_ref: tControl;

        begin // generate_table_recursive
          with p_c_panel do
          begin
            display('> generate_table_recursive '+ Name);

            l_panel_count:= 0;
            l_total_width:= 0;

            for l_control_index:= 0 to ControlCount- 1 do
            begin
              l_c_control_ref:= Controls[l_control_index];

              if l_c_control_ref.ClassName= 'TPanel'
                then begin
                    display('has_panel');
                    Inc(l_panel_count);

                    SetLength(l_panel_top_array, l_panel_count);
                    l_panel_top_array[l_panel_count- 1]:= l_c_control_ref.Top;

                    SetLength(l_panel_width_array, l_panel_count);
                    l_panel_width_array[l_panel_count- 1]:= l_c_control_ref.Width;
                    Inc(l_total_width, l_c_control_ref.Width);

                    SetLength(l_tPanel_array, l_panel_count);
                    l_tPanel_array[l_panel_count- 1]:= l_c_control_ref as tPanel;

                    SetLength(l_color_array, l_panel_count);
                    l_color_array[l_panel_count- 1]:= f_color((l_c_control_ref as tPanel).Color);
                  end
            end; // for l_control_index

            // -- if there is no panel, this is a "content surface"
            if l_panel_count= 0
              then generate_content
              else begin
                  // -- bgcolor="#FFBBFF"
                  l_color:= f_color(Color);

                  display(l_color);
                  generate_the_table;
                end;

            display('< generate_table_recursive');
          end; // with p_c_panel
        end; // generate_table_recursive

      begin // generate_page
        with p_c_design_panel do
        begin
          add_line_indent('<HTML>');
          add_line_indent('<BODY>');
          if ControlCount> 0
            then add_line_indent('<FORM method="POST" '
                + ' ACTION="http://127.0.0.1/scripts/training_information.exe"><BR>');

          generate_table_recursive(p_c_design_panel);

          if ControlCount> 0
            then unindent_add_line('</FORM>');
          unindent_add_line('</BODY>');
          unindent_add_line('</HTML>');
        end; // with p_c_design_panel
      end; // generate_page

    Destructor c_rad_html_generator.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_rad_html_generator
    end. // u_c_rad_html_generator


